@@ -1,5 +1,5 @@
 /*****************************************************************************
- * Copyright 2020 Reid Swanson
+ * Copyright 2020,2021 Reid Swanson
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,70 +18,70 @@
 #include <algorithm>
 #include <iostream>
 
-#include <boost/python.hpp>
-#include <boost/python/numpy.hpp>
-#include <boost/python/docstring_options.hpp>
+#include <pybind11/pybind11.h>
+#include <pybind11/numpy.h>
 #include <Eigen/Core>
 
 #include <Python.h>
 
 #include "gmix2/gmix.hpp"
 
-// See: https://stackoverflow.com/a/59694707/4971706
-namespace bp = boost::python;
-namespace np = boost::python::numpy;
+namespace py = pybind11;
+
+using namespace pybind11::literals;
 
 //-- region Utility Functions ---------------------------------------------------------------------------------------//
 template <typename Float>
 Eigen::Map<gmix::Array<Float>>
-ndarray_to_array(const np::ndarray& array) {
+ndarray_to_array(const py::array_t<Float>& array) {
     // Assume it's 1D
-    return Eigen::Map<gmix::Array<Float>>(reinterpret_cast<double*>(array.get_data()), 1, array.shape(0));
+    return Eigen::Map<gmix::Array<Float>>(const_cast<Float*>(array.data()), 1, array.shape(0));
 }
 
 template <typename Float>
 gmix::Array<Float>
-list_to_array(const bp::list& list) {
+list_to_array(const py::list& list) {
     // Assume it's 1D
-    const int size = bp::len(list);
+    const int size = py::len(list);
 
     gmix::Array<Float> array(size);
     for (int i = 0; i < size; ++i) {
-        array(i) = bp::extract<double>(list[i]);
+        array(i) = list[i].cast<Float>();
     }
     return array;
 }
 
 template <typename Float>
 Eigen::Map<gmix::Matrix<Float>>
-ndarray_to_matrix(const np::ndarray& array) {
+ndarray_to_matrix(const py::array_t<Float>& array) {
     const int n_rows = array.shape(0), n_cols = array.shape(1);
 
-    return Eigen::Map<gmix::Matrix<Float>>(reinterpret_cast<double*>(array.get_data()), n_rows, n_cols);
+    return Eigen::Map<gmix::Matrix<Float>>(const_cast<Float*>(array.data()), n_rows, n_cols);
 }
 
 template <typename Float>
-np::ndarray
+py::array_t<Float>
 matrix_to_array(const gmix::Matrix<Float> &matrix) {
     // Unfortunately, I don't think there's a way around copying the data for the return result;
-    const double * const data = matrix.data();
-    np::ndarray result = np::zeros(bp::make_tuple(matrix.rows(), matrix.cols()), np::dtype::get_builtin<double>());
-    std::copy(data, data + matrix.size(), reinterpret_cast<double*>(result.get_data()));
+    const Float * const data = matrix.data();
+    // py::array_t<Float> result = py::zeros(py::make_tuple(matrix.rows(), matrix.cols()), py::dtype::get_builtin<double>());
+    py::array_t<Float> result({matrix.rows(), matrix.cols()});
+    std::copy(data, data + matrix.size(), const_cast<Float*>(result.data()));
 
     return result;
 }
 
 template <typename Float>
-bp::list
+py::list
 mode_vector_to_list(const gmix::ModeVector<Float>& modes) {
-    bp::list result;
+    py::list result;
     for (int i = 0; i < modes.size(); ++i) {
         // For each mixture
-        bp::list mixture_result;
+        py::list mixture_result;
         for (int j = 0; j < modes[i].size(); ++j) {
             // For each mode in the mixture
             const auto& tuple = modes[i][j];
-            mixture_result.append(bp::make_tuple(std::get<0>(tuple), std::get<1>(tuple)));
+            mixture_result.append(py::make_tuple(std::get<0>(tuple), std::get<1>(tuple)));
         }
         result.append(std::move(mixture_result));
     }
@@ -92,8 +92,8 @@ mode_vector_to_list(const gmix::ModeVector<Float>& modes) {
 
 //-- region Wrapper Functions ---------------------------------------------------------------------------------------//
 //-- region Normal Distribution -------------------------------------------------------------------------------------//
-np::ndarray
-normal_pdf(double x, const np::ndarray& mean, const np::ndarray& std) {
+py::array_t<double>
+normal_pdf(double x, const py::array_t<double>& mean, const py::array_t<double>& std) {
     const auto& m = ndarray_to_matrix<double>(mean);
     const auto& s = ndarray_to_matrix<double>(std);
 
@@ -110,13 +110,17 @@ normal_pdf(double x, const np::ndarray& mean, const np::ndarray& std) {
  * @param std
  * @return
  */
-np::ndarray
-pdf(double x, const np::ndarray& weight, const np::ndarray& mean, const np::ndarray& std) {
+py::array_t<double>
+pdf(double x, const py::array_t<double>& weight, const py::array_t<double>& mean, const py::array_t<double>& std) {
     const auto& w = ndarray_to_matrix<double>(weight);
     const auto& m = ndarray_to_matrix<double>(mean);
     const auto& s = ndarray_to_matrix<double>(std);
 
-    return matrix_to_array(gmix::pdf<double>(x, w, m, s)).reshape(bp::make_tuple(-1));
+    //return matrix_to_array(gmix::pdf<double>(x, w, m, s)).reshape(py::make_tuple(-1));
+    auto array = matrix_to_array(gmix::pdf<double>(x, w, m, s));
+    array.resize({array.size()});
+
+    return array;
 }
 
 const char *pdf_docstring = "The `probability density function <https://en.wikipedia.org/wiki/Probability_density_function>`_ at x.\n\n"
@@ -132,8 +136,8 @@ const char *pdf_docstring = "The `probability density function <https://en.wikip
                             "Each row corresponds to a mixture and each column contains the PDF for the corresponding ``x`` value";
 
 
-np::ndarray
-pdf_multi_array(const np::ndarray& x, const np::ndarray& weight, const np::ndarray& mean, const np::ndarray& std) {
+py::array_t<double>
+pdf_multi_array(const py::array_t<double>& x, const py::array_t<double>& weight, const py::array_t<double>& mean, const py::array_t<double>& std) {
     const auto& i = ndarray_to_array<double>(x);
     const auto& w = ndarray_to_matrix<double>(weight);
     const auto& m = ndarray_to_matrix<double>(mean);
@@ -142,8 +146,8 @@ pdf_multi_array(const np::ndarray& x, const np::ndarray& weight, const np::ndarr
     return matrix_to_array(gmix::pdf<double>(i, w, m, s));
 }
 
-np::ndarray
-pdf_multi_list(const bp::list& x, const np::ndarray& weight, const np::ndarray& mean, const np::ndarray& std) {
+py::array_t<double>
+pdf_multi_list(const py::list& x, const py::array_t<double>& weight, const py::array_t<double>& mean, const py::array_t<double>& std) {
     const auto& i = list_to_array<double>(x);
     const auto& w = ndarray_to_matrix<double>(weight);
     const auto& m = ndarray_to_matrix<double>(mean);
@@ -160,13 +164,17 @@ pdf_multi_list(const bp::list& x, const np::ndarray& weight, const np::ndarray&
  * @param std
  * @return
  */
-np::ndarray
-cdf(double x, const np::ndarray& weight, const np::ndarray& mean, const np::ndarray& std) {
+py::array_t<double>
+cdf(double x, const py::array_t<double>& weight, const py::array_t<double>& mean, const py::array_t<double>& std) {
     const auto& w = ndarray_to_matrix<double>(weight);
     const auto& m = ndarray_to_matrix<double>(mean);
     const auto& s = ndarray_to_matrix<double>(std);
 
-    return matrix_to_array(gmix::cdf<double>(x, w, m, s)).reshape(bp::make_tuple(-1));
+    // return matrix_to_array(gmix::cdf<double>(x, w, m, s)).reshape(py::make_tuple(-1));
+    auto array = matrix_to_array(gmix::cdf<double>(x, w, m, s));
+    array.resize({array.size()});
+
+    return array;
 }
 
 const char *cdf_docstring = "The `cumulative distribution function <https://en.wikipedia.org/wiki/Cumulative_distribution_function>`_ at x.\n\n"
@@ -179,8 +187,8 @@ const char *cdf_docstring = "The `cumulative distribution function <https://en.w
                             "Note, Normal distributions are often characterized by their mean and variance, but the standard deviation is used in this toolkit.\n\n"
                             "**returns** (1 or 2d-ndarray): Returns the CDF value(s) for the input ``x`` in the same way as the :func:`.pdf`";
 
-np::ndarray
-cdf_multi_array(const np::ndarray& x, const np::ndarray& weight, const np::ndarray& mean, const np::ndarray& std) {
+py::array_t<double>
+cdf_multi_array(const py::array_t<double>& x, const py::array_t<double>& weight, const py::array_t<double>& mean, const py::array_t<double>& std) {
     const auto& i = ndarray_to_array<double>(x);
     const auto& w = ndarray_to_matrix<double>(weight);
     const auto& m = ndarray_to_matrix<double>(mean);
@@ -189,8 +197,8 @@ cdf_multi_array(const np::ndarray& x, const np::ndarray& weight, const np::ndarr
     return matrix_to_array(gmix::cdf<double>(i, w, m, s));
 }
 
-np::ndarray
-cdf_multi_list(const bp::list& x, const np::ndarray& weight, const np::ndarray& mean, const np::ndarray& std) {
+py::array_t<double>
+cdf_multi_list(const py::list& x, const py::array_t<double>& weight, const py::array_t<double>& mean, const py::array_t<double>& std) {
     const auto& i = list_to_array<double>(x);
     const auto& w = ndarray_to_matrix<double>(weight);
     const auto& m = ndarray_to_matrix<double>(mean);
@@ -211,12 +219,12 @@ cdf_multi_list(const bp::list& x, const np::ndarray& weight, const np::ndarray&
  * @param max_itr
  * @return
  */
-np::ndarray
+py::array_t<double>
 ppf(
     double p,
-    const np::ndarray& weight,
-    const np::ndarray& mean,
-    const np::ndarray& std,
+    const py::array_t<double>& weight,
+    const py::array_t<double>& mean,
+    const py::array_t<double>& std,
     const double lower_bound = -1e4,
     const double upper_bound = 1e4,
     const double tol = 1e-12,
@@ -226,9 +234,13 @@ ppf(
     const auto& m = ndarray_to_matrix<double>(mean);
     const auto& s = ndarray_to_matrix<double>(std);
 
-    return matrix_to_array(
-        gmix::ppf<double>(p, w, m, s, lower_bound, upper_bound, tol, max_itr)
-    ).reshape(bp::make_tuple(-1));
+//    return matrix_to_array(
+//        gmix::ppf<double>(p, w, m, s, lower_bound, upper_bound, tol, max_itr)
+//    ).reshape(py::make_tuple(-1));
+    auto array = matrix_to_array(gmix::ppf<double>(p, w, m, s, lower_bound, upper_bound, tol, max_itr));
+    array.resize({array.size()});
+
+    return array;
 }
 
 const char *ppf_docstring = "The `percent point function <https://en.wikipedia.org/wiki/Quantile_function>`_ (aka quantile or inverse cumulative distribution function)_ for probability p. "
@@ -247,12 +259,12 @@ const char *ppf_docstring = "The `percent point function <https://en.wikipedia.o
                             "**max_itr** (int): The maximum number of iterations before giving up.\n\n"
                             "**returns** (1 or 2d-ndarray): Returns the PPF value(s) for the input ``p`` in the same way as the :func:`.pdf`";
 
-np::ndarray
+py::array_t<double>
 ppf_multi_array(
-    const np::ndarray& p,
-    const np::ndarray& weight,
-    const np::ndarray& mean,
-    const np::ndarray& std,
+    const py::array_t<double>& p,
+    const py::array_t<double>& weight,
+    const py::array_t<double>& mean,
+    const py::array_t<double>& std,
     const double lower_bound = -1e4,
     const double upper_bound = 1e4,
     const double tol = 1e-12,
@@ -266,12 +278,12 @@ ppf_multi_array(
     return matrix_to_array(gmix::ppf<double>(i, w, m, s, lower_bound, upper_bound, tol, max_itr));
 }
 
-np::ndarray
+py::array_t<double>
 ppf_multi_list(
-    const bp::list& p,
-    const np::ndarray& weight,
-    const np::ndarray& mean,
-    const np::ndarray& std,
+    const py::list& p,
+    const py::array_t<double>& weight,
+    const py::array_t<double>& mean,
+    const py::array_t<double>& std,
     const double lower_bound = -1e4,
     const double upper_bound = 1e4,
     const double tol = 1e-12,
@@ -285,12 +297,16 @@ ppf_multi_list(
     return matrix_to_array(gmix::ppf<double>(i, w, m, s, lower_bound, upper_bound, tol, max_itr));
 }
 
-np::ndarray
-mean(const np::ndarray& weight, const np::ndarray& mean) {
+py::array_t<double>
+mean(const py::array_t<double>& weight, const py::array_t<double>& mean) {
     const auto& w = ndarray_to_matrix<double>(weight);
     const auto& m = ndarray_to_matrix<double>(mean);
 
-    return matrix_to_array(gmix::mean<double>(w, m)).reshape(bp::make_tuple(-1));
+    // return matrix_to_array(gmix::mean<double>(w, m)).reshape(py::make_tuple(-1));
+    auto array = matrix_to_array(gmix::mean<double>(w, m));
+    array.resize({array.size()});
+
+    return array;
 }
 
 const char *mean_docstring = "The mean <https://en.wikipedia.org/wiki/Mean>`_ value of the distribution(s).\n\n"
@@ -301,11 +317,11 @@ const char *mean_docstring = "The mean <https://en.wikipedia.org/wiki/Mean>`_ va
                              "**returns** (1d-ndarray): The mean value for each mixture";
 
 
-np::ndarray
+py::array_t<double>
 median(
-    const np::ndarray& weight,
-    const np::ndarray& mean,
-    const np::ndarray& std,
+    const py::array_t<double>& weight,
+    const py::array_t<double>& mean,
+    const py::array_t<double>& std,
     const double lower_bound = 1e-4,
     const double upper_bound = 1e4,
     const double tol = 1e-12,
@@ -315,9 +331,13 @@ median(
     const auto& m = ndarray_to_matrix<double>(mean);
     const auto& s = ndarray_to_matrix<double>(std);
 
-    return matrix_to_array(
-        gmix::median<double>(w, m, s, lower_bound, upper_bound, tol, max_itr)
-    ).reshape(bp::make_tuple(-1));
+//    return matrix_to_array(
+//        gmix::median<double>(w, m, s, lower_bound, upper_bound, tol, max_itr)
+//    ).reshape(py::make_tuple(-1));
+    auto array = matrix_to_array(gmix::median<double>(w, m, s, lower_bound, upper_bound, tol, max_itr));
+    array.resize({array.size()});
+
+    return array;
 }
 
 const char *median_docstring = "The `median <https://en.wikipedia.org/wiki/Median>`_ value of the distribution. "
@@ -332,11 +352,11 @@ const char *median_docstring = "The `median <https://en.wikipedia.org/wiki/Media
 
 
 
-bp::list
+py::list
 mode(
-    const np::ndarray& weight,
-    const np::ndarray& mean,
-    const np::ndarray& std,
+    const py::array_t<double>& weight,
+    const py::array_t<double>& mean,
+    const py::array_t<double>& std,
     const int max_itr = 1000,
     const double min_diff = 1e-4,
     const double min_grad = 1e-9
@@ -362,17 +382,21 @@ const char *mode_docstring = "The `mode(s) <https://en.wikipedia.org/wiki/Mode_(
                              "**returns** (a 2d list of tuples): For each mixture this returns a list of tuples. "
                              "The first value of each tuple is the ``x`` value of the mode and the second value is its density.";
 
-np::ndarray
+py::array_t<double>
 variance(
-    const np::ndarray& weight,
-    const np::ndarray& mean,
-    const np::ndarray& std
+    const py::array_t<double>& weight,
+    const py::array_t<double>& mean,
+    const py::array_t<double>& std
 ) {
     const auto& w = ndarray_to_matrix<double>(weight);
     const auto& m = ndarray_to_matrix<double>(mean);
     const auto& s = ndarray_to_matrix<double>(std);
 
-    return matrix_to_array(gmix::variance<double>(w, m, s)).reshape(bp::make_tuple(-1));
+    //return matrix_to_array(gmix::variance<double>(w, m, s)).reshape(py::make_tuple(-1));
+    auto array = matrix_to_array(gmix::variance<double>(w, m, s));
+    array.resize({array.size()});
+
+    return array;
 }
 
 const char *variance_docstring = "The `variance <https://en.wikipedia.org/wiki/Variance>`_ of the distribution. "
@@ -387,12 +411,12 @@ const char *variance_docstring = "The `variance <https://en.wikipedia.org/wiki/V
 
 // Boost Python does not like the name random for some reason and will fail to build
 // if the function has that name.
-np::ndarray
+py::array_t<double>
 gmix_random(
     size_t n,
-    const np::ndarray& weight,
-    const np::ndarray& mean,
-    const np::ndarray& std,
+    const py::array_t<double>& weight,
+    const py::array_t<double>& mean,
+    const py::array_t<double>& std,
     const int seed = -1
 ) {
     const auto& w = ndarray_to_matrix<double>(weight);
@@ -415,125 +439,92 @@ const char *random_docstring = "Generate `random <https://en.wikipedia.org/wiki/
 //-- endregion Mixture Distribution ---------------------------------------------------------------------------------//
 //-- endregion Wrapper Functions ------------------------------------------------------------------------------------//
 
-BOOST_PYTHON_FUNCTION_OVERLOADS(ppf_overloads, ppf, 4, 8)
-BOOST_PYTHON_FUNCTION_OVERLOADS(ppf_array_overloads, ppf_multi_array, 4, 8)
-BOOST_PYTHON_FUNCTION_OVERLOADS(ppf_list_overloads, ppf_multi_list, 4, 8)
-BOOST_PYTHON_FUNCTION_OVERLOADS(median_overloads, median, 3, 7)
-BOOST_PYTHON_FUNCTION_OVERLOADS(mode_overloads, mode, 3, 6)
-BOOST_PYTHON_FUNCTION_OVERLOADS(rand_overloads, gmix_random, 4, 5)
+//BOOST_PYTHON_FUNCTION_OVERLOADS(ppf_overloads, ppf, 4, 8)
+//BOOST_PYTHON_FUNCTION_OVERLOADS(ppf_array_overloads, ppf_multi_array, 4, 8)
+//BOOST_PYTHON_FUNCTION_OVERLOADS(ppf_list_overloads, ppf_multi_list, 4, 8)
+//BOOST_PYTHON_FUNCTION_OVERLOADS(median_overloads, median, 3, 7)
+//BOOST_PYTHON_FUNCTION_OVERLOADS(mode_overloads, mode, 3, 6)
+//BOOST_PYTHON_FUNCTION_OVERLOADS(rand_overloads, gmix_random, 4, 5)
 
-BOOST_PYTHON_MODULE(cgmix2) {
+PYBIND11_MODULE(cgmix2, m) {
     // NOTE: It is imperative that all numpy arrays created in gmix2
     // are in the default storage order ('C' / Row Major)
     Py_Initialize();
-    np::initialize();
+//    py::initialize();
 
-    bp::docstring_options doc_options(true, true, false);
-    bp::scope().attr("__doc__") = "All the functions listed and documented below in the ``cgmix2`` module are actually made avaliable in python by importing the ``gmix2`` module."
-                                        "I apologize for this confusion which is probably arising due to a misunderstanding of how to structure and name C++ extensions";
+    // py::docstring_options doc_options(true, true, false);
+    m.doc() = "All the functions listed and documented below in the ``cgmix2`` module are actually made available "
+              "in python by importing the ``gmix2`` module. I apologize for this confusion which is probably "
+              "arising due to a misunderstanding of how to structure and name C++ extensions";
 
-    bp::def("normal_pdf", normal_pdf);
-
-    bp::def("pdf", pdf, bp::args("x", "weights", "means", "stddevs"), pdf_docstring);
-    bp::def("pdf", pdf_multi_array, bp::args("x", "weights", "means", "stddevs"));
-    bp::def("pdf", pdf_multi_list, bp::args("x", "weights", "means", "stddevs"));
-    bp::def("cdf", cdf, bp::args("x", "weights", "means", "stddevs"), cdf_docstring);
-    bp::def("cdf", cdf_multi_array, bp::args("x", "weights", "means", "stddevs"));
-    bp::def("cdf", cdf_multi_list, bp::args("x", "weights", "means", "stddevs"));
-    bp::def(
+    m.def("normal_pdf", normal_pdf);
+    m.def("pdf", &pdf, pdf_docstring, "x"_a, "weights"_a, "means"_a, "stddevs"_a);
+    m.def("pdf", &pdf_multi_array, "x"_a, "weights"_a, "means"_a, "stddevs"_a);
+    m.def("pdf", &pdf_multi_list, "x"_a, "weights"_a, "means"_a, "stddevs"_a);
+    m.def("cdf", &cdf, cdf_docstring, "x"_a, "weights"_a, "means"_a, "stddevs"_a);
+    m.def("cdf", &cdf_multi_array, "x"_a, "weights"_a, "means"_a, "stddevs"_a);
+    m.def("cdf", &cdf_multi_list, "x"_a, "weights"_a, "means"_a, "stddevs"_a);
+    m.def(
         "ppf",
-        ppf,
-        ppf_overloads(
-            (
-                bp::args("p"),
-                bp::args("weight"),
-                bp::args("mean"),
-                bp::args("std"),
-                bp::args("lower_bound") = -1e4,
-                bp::args("upper_bound") = 1e4,
-                bp::args("tol") = 1e-12,
-                bp::args("max_itr") = 100
-            ),
-            ppf_docstring
-        )
-    );
-    bp::def(
+        &ppf,
+        ppf_docstring,
+        "p"_a,
+        "weight"_a,
+        "mean"_a,
+        "std"_a,
+        "lower_bound"_a = -1e4,
+        "upper_bound"_a = 1e4,
+        "tol"_a = 1e-12,
+        "max_itr"_a = 100
+        );
+    m.def(
         "ppf",
-        ppf_multi_array,
-        ppf_array_overloads(
-            (
-                bp::args("p"),
-                bp::args("weight"),
-                bp::args("mean"),
-                bp::args("std"),
-                bp::args("lower_bound") = -1e4,
-                bp::args("upper_bound") = 1e4,
-                bp::args("tol") = 1e-12,
-                bp::args("max_itr") = 100
-            )
-        )
+        &ppf_multi_array,
+        "p"_a,
+        "weight"_a,
+        "mean"_a,
+        "std"_a,
+        "lower_bound"_a = -1e4,
+        "upper_bound"_a = 1e4,
+        "tol"_a = 1e-12,
+        "max_itr"_a = 100
     );
-    bp::def(
+    m.def(
         "ppf",
-        ppf_multi_list,
-        ppf_list_overloads(
-            (
-                bp::args("p"),
-                bp::args("weight"),
-                bp::args("mean"),
-                bp::args("std"),
-                bp::args("lower_bound") = -1e4,
-                bp::args("upper_bound") = 1e4,
-                bp::args("tol") = 1e-12,
-                bp::args("max_itr") = 100
-            )
-        )
+        &ppf_multi_list,
+        "p"_a,
+        "weight"_a,
+        "mean"_a,
+        "std"_a,
+        "lower_bound"_a = -1e4,
+        "upper_bound"_a = 1e4,
+        "tol"_a = 1e-12,
+        "max_itr"_a = 100
     );
-    bp::def("mean", mean, bp::args("weights", "means"), mean_docstring);
-    bp::def(
+    m.def("mean", &mean, mean_docstring, "weights"_a, "means"_a);
+    m.def(
         "median",
-        median,
-        median_overloads(
-            (
-                bp::args("weight"),
-                bp::args("mean"),
-                bp::args("std"),
-                bp::args("lower_bound") = -1e4,
-                bp::args("upper_bound") = 1e4,
-                bp::args("tol") = 1e-12,
-                bp::args("max_itr") = 100
-            ),
-            median_docstring
-        )
+        &median,
+        median_docstring,
+        "weights"_a,
+        "means"_a,
+        "std"_a,
+        "lower_bound"_a = -1e4,
+        "upper_bound"_a = 1e4,
+        "tol"_a = 1e-12,
+        "max_itr"_a = 100
     );
-    bp::def(
+    m.def(
         "mode",
-        mode,
-        mode_overloads(
-            (
-                bp::args("weight"),
-                bp::args("mean"),
-                bp::args("std"),
-                bp::args("max_itr") = 1000,
-                bp::args("min_diff") = 1e-4,
-                bp::args("min_grad") = 1e-9
-            ),
-            mode_docstring
-        )
-    );
-    bp::def("variance", variance, bp::args("weignts", "means", "stddevs"), variance_docstring);
-    bp::def(
-        "random",
-        gmix_random,
-        rand_overloads(
-            (
-                bp::args("n"),
-                bp::args("weight"),
-                bp::args("mean"),
-                bp::args("std"),
-                bp::args("seed") = 1
-            ),
-            random_docstring
-        )
+        &mode,
+        mode_docstring,
+        "weight"_a,
+        "mean"_a,
+        "std"_a,
+        "max_itr"_a = 1000,
+        "min_diff"_a = 1e-4,
+        "min_grad"_a = 1e-9
     );
+    m.def("variance", &variance, variance_docstring, "weights"_a, "means"_a, "stddevs"_a);
+    m.def("random", &gmix_random, random_docstring, "n"_a, "weight"_a, "mean"_a, "std"_a, "seed"_a = 1);
 }
